package com.fr.plugin.cpic.filter;

import com.fanruan.api.json.JSONKit;
import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.ServerConfig;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.bean.authority.PrivilegeDetailBean;
import com.fr.decision.webservice.bean.entry.DirectoryBean;
import com.fr.decision.webservice.bean.entry.EntryBean;
import com.fr.decision.webservice.v10.entry.EntryService;
import com.fr.plugin.cpic.db.bean.CpicEntryBean;
import com.fr.plugin.cpic.db.bean.CpicHomeBean;
import com.fr.plugin.cpic.db.service.CpicEntryService;
import com.fr.plugin.cpic.db.service.CpicHomeService;
import com.fr.plugin.transform.ExecuteFunctionRecord;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.third.org.apache.commons.collections4.CollectionUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.*;

/**
 * 目录拦截器
 */
@FunctionRecorder(localeKey = "EntryFilter")
public class EntryFilter extends AbstractGlobalRequestFilterProvider {


    @Override
    public String filterName() {
        return "EntryFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    @ExecuteFunctionRecord
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            // 判断是否拦截
//            String originalUrl = WebUtils.getOriginalURL(req);
            String requestURI = req.getRequestURI();
            if(!requestURI.endsWith("/v10/view/entry/tree") || !req.getMethod().equals("GET")){
                filterChain.doFilter(req, res);
                return;
            }

            // 判断请求来源
            String referer = req.getHeader("Referer");
            if (StringKit.isBlank(referer)) {
                filterChain.doFilter(req, res);
                return;
            }
            String[] refererSpArr = referer.split("/");
            String h1 = "";
            String h2 = "";
            if (refererSpArr.length >= 2) {
                h1 = refererSpArr[refererSpArr.length - 2];
                h2 = refererSpArr[refererSpArr.length - 1];
                if (StringKit.isBlank(h1) || !StringKit.equals("home", h1)) {
                    filterChain.doFilter(req, res);
                    return;
                }
                if (StringKit.isBlank(h2)) {
                    filterChain.doFilter(req, res);
                    return;
                }
            }

            // 查询对应home的path是否存在
            CpicHomeBean homeBean = CpicHomeService.findByPath(h2, "");
            if (null == homeBean) {
                filterChain.doFilter(req, res);
                return;
            }

//            filterChain.doFilter(req, res);

            // 当前门户配置的目录
            List<CpicEntryBean> cpicEntryBeanList = CpicEntryService.findList(homeBean.getId());
            LogKit.debug("门户目录: "+JSONKit.createJSONArray(cpicEntryBeanList).toString());
            // 当前用户配置的目录
            List<EntryBean> entryBeanList = EntryService.getInstance().getEntryTree(req);
            LogKit.debug("用户目录: "+JSONKit.createJSONArray(entryBeanList).toString());

            // 其中一个为空时交集一定为空，直接返回空
            if (CollectionUtils.isEmpty(cpicEntryBeanList) || CollectionUtils.isEmpty(entryBeanList)) {
                Map<String, Object> result = new HashMap<>();
                result.put("data", Collections.EMPTY_LIST);
                PrintWriter printer = res.getWriter();
                printer.write(JSONKit.create(result).toString());
                printer.flush();
                printer.close();
            }

            // 取交集
            List<EntryBean> resultList = new ArrayList<>();
            for (EntryBean entry : entryBeanList) {
                CpicEntryBean cpicEntryBean = cpicEntryBeanList.stream().filter(item -> item.getEntryId().equals(entry.getId())).findFirst().orElse(null);
                if (null != cpicEntryBean) {
                    resultList.add(entry);
                }
            }

            EntryBean root = entryBeanList.stream().filter(item -> "decision-directory-root".equals(item.getId())).findFirst().orElse(null);
            if(null == root){
                // 手动创建一个根目录
                root = new DirectoryBean();
                root.setId("decision-directory-root");
                root.setText("Dec-Entry_Management");
                root.setDeviceType(7);
                root.setEntryType(3);
                root.setSortIndex(0);
                root.setIsParent(true);
                root.setOpen(false);
                root.setParentDeviceType(7);
                List<PrivilegeDetailBean> privilegeDetailBeanList = new ArrayList<>();
                PrivilegeDetailBean privilegeDetailBean = new PrivilegeDetailBean();
                privilegeDetailBean.setPrivilegeType(1);
                privilegeDetailBean.setPrivilegeValue(3);
                root.setPrivilegeDetailBeanList(privilegeDetailBeanList);
            }
            resultList.add(root);

            LogKit.debug("交集目录: "+JSONKit.createJSONArray(resultList).toString());

            res.setContentType("text/html; charset="+ ServerConfig.getInstance().getServerCharset());
            // 返回数据
            Map<String, Object> result = new HashMap<>();
            result.put("data", resultList);
            PrintWriter printer = res.getWriter();
            printer.write(JSONKit.create(result).toString());
            printer.flush();
            printer.close();
        } catch (Exception e) {
            LogKit.error(e.getMessage());
            e.printStackTrace();
        }
    }
}
