package com.fr.plugin.cpic.filter;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.plugin.cpic.db.bean.CpicHomeBean;
import com.fr.plugin.cpic.db.service.CpicHomeService;
import com.fr.plugin.transform.ExecuteFunctionRecord;
import com.fr.plugin.transform.FunctionRecorder;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 登出拦截器
 */
@FunctionRecorder(localeKey = "LogoutFilter")
public class LogoutFilter extends AbstractGlobalRequestFilterProvider {


    @Override
    public String filterName() {
        return "LogoutFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    @ExecuteFunctionRecord
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            // 判断是否拦截
//            String originalUrl = WebUtils.getOriginalURL(req);
            String requestURI = req.getRequestURI();
            if (!requestURI.endsWith("/logout") || !req.getMethod().equals("POST")) {
                filterChain.doFilter(req, res);
                return;
            }

            // 判断请求来源
            String referer = req.getHeader("Referer");
            if (StringKit.isBlank(referer)) {
                filterChain.doFilter(req, res);
                return;
            }
            String[] refererSpArr = referer.split("/");
            String h1 = "";
            String h2 = "";
            if (refererSpArr.length >= 2) {
                h1 = refererSpArr[refererSpArr.length - 2];
                h2 = refererSpArr[refererSpArr.length - 1];
                if (StringKit.isBlank(h1) || !StringKit.equals("home", h1)) {
                    filterChain.doFilter(req, res);
                    return;
                }
                if (StringKit.isBlank(h2)) {
                    filterChain.doFilter(req, res);
                    return;
                }
            }

            // 查询对应home的path是否存在
            CpicHomeBean homeBean = CpicHomeService.findByPath(h2, "");
            if (null == homeBean) {
                filterChain.doFilter(req, res);
                return;
            }

            // 登出
            LoginService.getInstance().logout(req, res);

            // 跳转到门户首页
            this.sendRedirect(res, referer);
            return;
        } catch (Exception e) {
            LogKit.error(e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * 重定向 1
     *
     * @param res
     * @param url
     */
    private void sendRedirect(HttpServletResponse res, String url) {
        res.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }
}
