package com.fr.plugin.cpic.web.controller;

import com.fanruan.api.util.StringKit;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.bean.config.ThemeConfigBean;
import com.fr.decision.webservice.v10.config.ConfigService;
import com.fr.plugin.cpic.db.bean.CpicHomeBean;
import com.fr.plugin.cpic.db.service.CpicHomeService;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.RequestBody;
import com.fr.third.springframework.web.bind.annotation.RequestMapping;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 回调接口
 */
@Controller
@LoginStatusChecker(required = false)
@RequestMapping(value = "/cpic")
public class CpicHomeController {

    @RequestMapping(value = "/home", method = RequestMethod.GET)
    public void index(HttpServletRequest request, HttpServletResponse response) throws Exception {
        HashMap hashMap = new HashMap();
        hashMap.put("loginUser", "xx");
        hashMap.put("callBack", "xx");
        WebUtils.writeOutTemplate("/com/fr/plugin/cpic/web/html/home.html", response, hashMap);
    }

    @RequestMapping(value = "/home/list", method = RequestMethod.GET)
    @ResponseBody
    public Response getAll(HttpServletRequest request, HttpServletResponse response) throws Exception {
        try {
            List<CpicHomeBean> list = CpicHomeService.findList();
            Map<String, Object> result = new HashMap<>();
            result.put("homeList", list);
            return Response.ok(result);
        } catch (Exception e) {
            return Response.error("-1", "error:" + e.getMessage());
        }
    }

    @RequestMapping(value = "/home/themes", method = RequestMethod.GET)
    @ResponseBody
    public Response getAllTheme(HttpServletRequest request, HttpServletResponse response) throws Exception {
        try {
            return Response.ok(ConfigService.getInstance().getAllThemes());
        } catch (Exception e) {
            return Response.error("-1", "error:" + e.getMessage());
        }
    }

    @RequestMapping(value = "/home/edit", method = RequestMethod.POST)
    @ResponseBody
    public Response saveOrUpdate(HttpServletRequest request, HttpServletResponse response, @RequestBody CpicHomeBean cpicHomeBean) throws Exception {
        try {
            // 校验输入
            if (StringKit.isBlank(cpicHomeBean.getName())) {
                return Response.error("-1", "请输入门户名称！");
            }
            if (StringKit.isBlank(cpicHomeBean.getPath())) {
                return Response.error("-1", "请输入门户地址！");
            }

            // 校验重名
            CpicHomeBean nameBean = CpicHomeService.findByName(cpicHomeBean.getName(), cpicHomeBean.getId());
            if (null != nameBean) {
                return Response.error("-1", "门户名称重复,请修改后重试！");
            }
            CpicHomeBean pathBean = CpicHomeService.findByPath(cpicHomeBean.getPath(), cpicHomeBean.getId());
            if (null != pathBean) {
                return Response.error("-1", "门户地址重复，请修改后重试！");
            }

            List<ThemeConfigBean> themeList = ConfigService.getInstance().getAllThemes();
            ThemeConfigBean themeConfigBean = themeList.stream().filter(item -> item.getThemeId().equals(cpicHomeBean.getThemeId())).findFirst().orElse(null);
            if (null != themeConfigBean) {
                cpicHomeBean.setThemeName(themeConfigBean.getName());
            }
            if ("classic".equals(cpicHomeBean.getThemeId())) {
                cpicHomeBean.setThemeName("经典");
            }
            if ("modern".equals(cpicHomeBean.getThemeId())) {
                cpicHomeBean.setThemeName("扁平化");
            }

            // 保存
            CpicHomeService.saveOrUpdate(cpicHomeBean);

            // 返回成功的数据
            CpicHomeBean result = CpicHomeService.findByName(cpicHomeBean.getName(), "");
            return Response.ok(result);
        } catch (Exception e) {
            return Response.error("-1", "error:" + e.getMessage());
        }
    }

    @RequestMapping(value = "/home/del", method = RequestMethod.POST)
    @ResponseBody
    public Response delete(HttpServletRequest request, HttpServletResponse response, @RequestBody CpicHomeBean cpicHomeBean) throws Exception {
        try {
            // 删除
            CpicHomeService.logicDel(cpicHomeBean.getId());
            return Response.success();
        } catch (Exception e) {
            return Response.error("-1", "error:" + e.getMessage());
        }
    }

}
