package com.fr.plugin.cpic.web.controller;

import com.fanruan.api.log.LogKit;
import com.fr.base.ServerConfig;
import com.fr.decision.authority.data.User;
import com.fr.decision.inject.DecisionInjectExtraInfoBuilder;
import com.fr.decision.inject.DefaultDecisionInjectNodes;
import com.fr.decision.inject.node.DecisionInjectNode;
import com.fr.decision.inject.node.DecisionInjectNodeManager;
import com.fr.decision.service.DecisionService;
import com.fr.decision.web.MainComponent;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.plugin.cpic.db.bean.CpicHomeBean;
import com.fr.plugin.cpic.db.bean.CpicUserBean;
import com.fr.plugin.cpic.db.service.CpicHomeService;
import com.fr.plugin.cpic.db.service.CpicUserService;
import com.fr.plugin.cpic.web.custom.CustomDecisionErrorInjectNode;
import com.fr.plugin.cpic.web.custom.CustomDecisionSystemInjectNode;
import com.fr.plugin.cpic.web.custom.CustomDecisionUserInjectNode;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.PathVariable;
import com.fr.third.springframework.web.bind.annotation.RequestMapping;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;
import com.fr.web.Browser;
import com.fr.web.struct.AtomBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;

/**
 * 页面
 */
@Controller
//@LoginStatusChecker(required = false)
@RequestMapping(value = "/home")
public class CpicHomeIndexController {

    @RequestMapping(
            value = {"/{path}"},
            produces = {"text/html"}
    )
    @ResponseBody
    @LoginStatusChecker(
            tokenResource = TokenResource.COOKIE
    )
    public String home(HttpServletRequest request, HttpServletResponse response, @PathVariable("path") String path) throws Exception {
        HashMap params = new HashMap();
        params.put("charset", ServerConfig.getInstance().getServerCharset());
        // 根据path查询
        CpicHomeBean home = CpicHomeService.findByPath(path, "");
        if (null == home) {
            DecisionInjectNode[] nodes = new DecisionInjectNode[]{
                    new CustomDecisionErrorInjectNode("NN", "", "门户地址[" + path + "]不存在")
            };
            return DecisionInjectNodeManager.box(
                    DecisionInjectExtraInfoBuilder.builder().request(request).user(DecisionService.getInstance().authority().userService().getUserByRequestCookie(request)).pagePath("/com/fr/plugin/cpic/web/html/error.html").pathGroup(AtomBuilder.create().buildAssembleFilePath(Browser.resolve(request), MainComponent.KEY)).build(),
                    nodes);
        }
        User user = DecisionService.getInstance().authority().userService().getUserByRequestCookie(request);
        // 查询当前用户是否有登录权限
        CpicUserBean cpicUserBean = CpicUserService.findHomeUserByUserId(home.getId(), user.getId());
        // 没有权限的用户跳转到提示页面
        if (null == cpicUserBean) {
            DecisionInjectNode[] nodes = new DecisionInjectNode[]{
                    new CustomDecisionErrorInjectNode("YY", home.getPath(), "登录用户[" + user.getUserName() + "]没有门户[" + home.getName() + "]登录权限")
            };
            return DecisionInjectNodeManager.box(
                    DecisionInjectExtraInfoBuilder.builder().request(request).user(DecisionService.getInstance().authority().userService().getUserByRequestCookie(request)).pagePath("/com/fr/plugin/cpic/web/html/error.html").pathGroup(AtomBuilder.create().buildAssembleFilePath(Browser.resolve(request), MainComponent.KEY)).build(),
                    nodes);
        }

        // 主页
        LogKit.debug("门户首页地址：", home.getMhpath());
        CustomDecisionUserInjectNode customUserNode = new CustomDecisionUserInjectNode(home.getMhpath());

        // 主题
        LogKit.debug("门户主题：", home.getThemeId());
        CustomDecisionSystemInjectNode customSystemNode = new CustomDecisionSystemInjectNode(home.getThemeId());

        // 生成数据
        DecisionInjectNode[] nodes = new DecisionInjectNode[]{
                DefaultDecisionInjectNodes.scriptInfo(),
                DefaultDecisionInjectNodes.styleInfo(),
                DefaultDecisionInjectNodes.charsetInfo(),
//                DefaultDecisionInjectNodes.systemInfo(),
                customSystemNode,
//                DefaultDecisionInjectNodes.userInfo(),
                customUserNode,
                DefaultDecisionInjectNodes.titleInfo()
        };

        return DecisionInjectNodeManager.box(
                DecisionInjectExtraInfoBuilder.builder().request(request).user(DecisionService.getInstance().authority().userService().getUserByRequestCookie(request)).pagePath("/com/fr/plugin/cpic/web/html/resources/index.html").pathGroup(AtomBuilder.create().buildAssembleFilePath(Browser.resolve(request), MainComponent.KEY)).build(),
                nodes);

    }



}
