package com.fr.plugin.cpic.web.custom;

import com.fr.decision.config.SystemConfig;
import com.fr.decision.inject.DecisionInjectExtraInfoBuilder;
import com.fr.decision.inject.node.impl.AbstractDecisionInjectNode;
import com.fr.decision.service.DecisionService;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.entry.EntryService;
import com.fr.decision.webservice.v10.entry.homepage.HomePageInfo;
import com.fr.decision.webservice.v10.module.ManagerModuleService;
import com.fr.security.ipcheck.IPMatchHandler;
import com.fr.stable.core.UUID;
import com.fr.third.springframework.web.context.request.RequestContextHolder;
import com.fr.third.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CustomDecisionUserInjectNode extends AbstractDecisionInjectNode {

    private String homepage;

    public String getHomepage() {
        return homepage;
    }

    public void setHomepage(String homepage) {
        this.homepage = homepage;
    }

    public CustomDecisionUserInjectNode(String homepage){
        this.setHomepage(homepage);
    }

    public String name() {
        return "personal";
    }

    @Override
    protected Map<String, Object> calculate(DecisionInjectExtraInfoBuilder var1) throws Exception {
        HashMap var2 = new HashMap();
        if (var1.user() != null) {
            var2.put("username", var1.user().getUserName());
            var2.put("userId", var1.user().getId());
            var2.put("displayName", var1.user().getDisplayName());
            var2.put("userExtraProps", DecisionService.getInstance().authority().userService().getUserExtraProperties(var1.user()));
            var2.put("creationType", var1.user().getCreationType().toInteger());
            var2.put("realName", var1.user().getRealName());
            HttpServletRequest var3 = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            if (!var1.isIgnoreDB()) {
                var2.put("homepage", EntryService.getInstance().getHomePageUrl(var1.user().getId()));
                var2.put("modules", this.checkAdminLoginIp(var3) ? ManagerModuleService.getInstance().getAllDecisionMgrModules(var1.user().getId(), "") : new ArrayList());
            }

            // 数据门户
            List<HomePageInfo> homePageInfoList = new ArrayList<>();
            HomePageInfo homePageInfo = new HomePageInfo();
            homePageInfo.setId(UUID.randomUUID().toString().toLowerCase());
            homePageInfo.setHomePageType(2);
            homePageInfo.setText("首页");
            homePageInfo.setPcURL(this.getHomepage());
            homePageInfoList.add(homePageInfo);
            var2.put("homepage", homePageInfoList);

        }

        return var2;
    }

    private boolean checkAdminLoginIp(HttpServletRequest var1) {
        return this.checkAdminLoginIp(WebServiceUtils.getIpInfoFromRequest(var1));
    }

    private boolean checkAdminLoginIp(String var1) {
        if (SystemConfig.getInstance().getEnableWhiteVerify()) {
            String[] var2 = SystemConfig.getInstance().getWhiteIps();
            if (IPMatchHandler.noneMatch(var1, IPMatchHandler.addLocalIP(var2))) {
                return false;
            }
        }

        return true;
    }

}
